#pragma once
#ifndef STOOLS_H
#define STOOLS_H

#include <windows.h>
#include <fstream>
#include <string>
#include <stdio.h>
#include <iostream>

using namespace std;

struct tsym
{
   unsigned char s;
   int attr;
};

typedef struct tsym tsym;

typedef tsym tpage[25][80];

const int screenw=80, screenh=25;

bool gotoXY(int x, int y);
bool set_color(int attr);
bool set_default_color();
bool PutXY(int x, int y, char c, int attr);
bool PutsXY(int x, int y, char* s, int attr);
bool PutSym(tsym sym, int x, int y);
bool PageOut(tpage page);
bool PageOut2(tpage page);
bool BufferOut(CHAR_INFO buf[2000], SMALL_RECT rect);
bool SetCursor(int size, bool visible);
bool LoadFileToPage(char* fname, tpage &page);
bool SavePageToFile(char* fname, tpage page);

bool gotoXY(int x, int y) 
{
if(x<0 || x>80 || y<0 || y>24 ) return false;
HANDLE handle= GetStdHandle(STD_OUTPUT_HANDLE);
COORD CursorPosition; 
CursorPosition.X = x; // Locates column
CursorPosition.Y = y; // Locates Row
SetConsoleCursorPosition(handle,CursorPosition); // Sets position for next thing to be printed 
return true;
}

bool set_color(int attr)
{
if(attr<0 || attr>255) return false;
 HANDLE handle= GetStdHandle(STD_OUTPUT_HANDLE);
 SetConsoleTextAttribute( handle, attr);
 return true;
} 

bool set_default_color()
{
    return set_color(7);
}

bool PutXY(int x, int y, char c, int attr)
{
     if(attr>=0) set_color(attr);
     else set_default_color();
     gotoXY(x, y);
     printf("%c",c);
     set_default_color();
	 return true; 
}

bool PutsXY(int x, int y, char* s, int attr)
{
     if(attr>=0) set_color(attr);
     else set_default_color();
     gotoXY(x, y);
     printf(s);
     set_default_color();
	 return true; 
}

bool PutSym(tsym sym, int x, int y)
{
     PutXY(x, y, sym.s, sym.attr);
	 return true;
}

bool PageOut(tpage page)
{
     for(int j=0; j<screenh; j++)
     {
       for(int i=0; i<screenw; i++)
       {
         PutSym(page[j][i],i,j);
         
       }
       //gotoXY(0, 0); printf("%d",j);
      }

	 return true;
}

bool PageOut2(tpage page)
{
    const int row = 25; 
 	const int col = 80; 
 	SMALL_RECT writeArea={0,0,79,24};
	CHAR_INFO consBuffer[col * row];
	
	for(int j=0; j<25; j++)
	{	 
		for (int i = 0; i<80; ++i) { 
			consBuffer[j*80+i].Char.UnicodeChar = page[j][i].s; 
			consBuffer[j*80+i].Attributes = page[j][i].attr;
		}
		
	}
	
	BufferOut(consBuffer, writeArea);
	return true;
}

bool BufferOut(CHAR_INFO buf[2000], SMALL_RECT rect)
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
 	COORD charPosition={0,0}; 
 	COORD bufferSize = {80,25}; 
 	WriteConsoleOutput(hConsole, buf, bufferSize, charPosition, &rect);
	return true; 
}

bool SetCursor(int size, bool visible)
{
HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
    CONSOLE_CURSOR_INFO CursoInfo;
    CursoInfo.dwSize = 1;         /* The size of caret */
    CursoInfo.bVisible = false;   /* Caret is visible? */
    SetConsoleCursorInfo(hConsole, &CursoInfo);
    return true;
}

bool LoadFileToPage(char* fname, tpage &page)
{
    int i,j,k,n;
    string sLine;
	
	string delimiter = "\t";

	size_t pos = 0;
	string token;
	i=j=0;
	n=1;
	
	for(j=0; j<25; j++)
		for(i=0; i<80; i++)
		{
			page[j][i].s = 49;
		    page[j][i].attr = 7;
		}

	ifstream is(fname);
	//if(is.is_open())
	//{
	i=j=0;
	while (!is.eof()) {	
	  getline(is,sLine);	

	  while ((pos = sLine.find(delimiter)) != std::string::npos) {
		    token = sLine.substr(0, pos);
		    k=atoi(token.c_str());
		    if(i<160 && j<25 && k<256)
		    {
		    	if(n==1) page[j][(int)i/2].s = k;
		    	if(n==2) page[j][(int)i/2].attr = k;
			}
		    
		    sLine.erase(0, pos + delimiter.length());
		    i++;
		    n=3-n;
		    
		}
		token = sLine;
		if(i<160 && j<25 && k<256)
	    {
	    	if(n==1) page[j][(int)i/2].s = k;
	    	if(n==2) page[j][(int)i/2].attr = k;
		}
	  	i=0;
	  	n=1;
	   //split up the line
	j++;
	}
	
	is.close();
	return true;
	//}
	//else return false;
}

bool SavePageToFile(char* fname, tpage page)
{
	fstream myfile;
	myfile.open(fname, ios::out);	
	if (myfile.is_open())
	{
		for(int j=0; j<25; j++){
			for(int i=0; i<80; i++)
			{
				myfile << (int) page[j][i].s<<'\t'<<page[j][i].attr;
				if(i<79) myfile << '\t';
			}
			if(j<24) myfile << '\n';
		}
		myfile.close();
		return true;
	}
	else return false;
}
#endif
